package main

import (
	"os"
	"os/signal"

	"github.com/gofiber/fiber/v2"
	"github.com/gofiber/swagger"

	"gofiber.study.skcks.cn/common/errorx"
	"gofiber.study.skcks.cn/common/response"
	"gofiber.study.skcks.cn/logger"

	_ "gofiber.study.skcks.cn/docs"
)

var log = logger.Logger()

// @title GoFiber Study API
// @version 1.0
// @description 基于 Fiber 框架的 swagger
// @termsOfService http://swagger.io/terms/
// @contact.name Shikong
// @contact.email 919411476@qq.com
// @license.name Apache 2.0
// @license.url http://www.apache.org/licenses/LICENSE-2.0.html
// @BasePath /
func main() {
	app := fiber.New(fiber.Config{
		Prefork:       true,
		CaseSensitive: true,
		StrictRouting: false,
		ServerHeader:  "SkServer",
		ErrorHandler: func(ctx *fiber.Ctx, err error) error {
			code := fiber.StatusInternalServerError

			if e, ok := err.(*fiber.Error); ok {
				code = e.Code
			}

			err = ctx.Status(code).JSON(errorx.NewDefaultError(err.Error()))
			if err != nil {
				return ctx.Status(fiber.StatusInternalServerError).SendString("Internal Server Error")
			}

			return nil
		},
	})

	app.Get("/swagger/*", swagger.New(swagger.Config{
		DeepLinking: false,
		// Expand ("list") or Collapse ("none") tag groups by default
		DocExpansion: "full",
	}))

	helloWorld(app)

	app.Get("/routes", func(ctx *fiber.Ctx) error {
		return ctx.JSON(response.NewResponse(app.Stack()))
	})

	go func() {
		if err := app.Listen(":3000"); err != nil {
			log.Fatalf("致命错误: %s", err)
		}
	}()

	quit := make(chan os.Signal)
	signal.Notify(quit, os.Interrupt)
	<-quit
	log.Infof("关闭服务")
}

// helloWorld
//
// @Summary      HelloWorld
// @Description  简单的 HelloWorld 示例
// @Tags         HelloWorld
// @Accept       json
// @Produce      json
// @Success      200  {string}  string
// @Failure      400  {object}  errorx.CodeErrorResponse
// @Failure      404  {object}  errorx.CodeErrorResponse
// @Failure      500  {object}  errorx.CodeErrorResponse
// @Router       / [get]
func helloWorld(app *fiber.App) {
	app.Get("/", func(c *fiber.Ctx) error {
		log.Infof("\n%s", c.Request().String())
		return c.JSON(response.NewResponse("Hello, World 👋!"))
	})
}
